package resumable.js.upload;

package .var.folders.cl.9twcgjfx4vsg5zl73c6wxmvm0000gn.T;

import cpcns.base.configuration.AppConfiguration;
import java.io.File;
import java.io.IOException;
import java.io.RandomAccessFile;
import javax.servlet.ServletException;
import javax.servlet.ServletInputStream;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import resumable.js.upload.HttpUtils;
import resumable.js.upload.ResumableInfo;
import resumable.js.upload.ResumableInfoStorage;

public class UploadServlet extends HttpServlet {
  public static final String UPLOAD_DIR = AppConfiguration.getConfigValue("OACaiJiPath") + "/upload_dir/";
  
  protected void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
    int resumableChunkNumber = getResumableChunkNumber(request);
    ResumableInfo info = getResumableInfo(request);
    RandomAccessFile raf = new RandomAccessFile(info.resumableFilePath, "rw");
    raf.seek((resumableChunkNumber - 1) * info.resumableChunkSize);
    ServletInputStream servletInputStream = request.getInputStream();
    long readed = 0L;
    long content_length = request.getContentLength();
    byte[] bytes = new byte[102400];
    while (readed < content_length) {
      int r = servletInputStream.read(bytes);
      if (r < 0)
        break; 
      raf.write(bytes, 0, r);
      readed += r;
    } 
    raf.close();
    info.uploadedChunks.add(new ResumableInfo.ResumableChunkNumber(resumableChunkNumber));
    if (info.checkIfUploadFinished()) {
      ResumableInfoStorage.getInstance().remove(info);
      response.getWriter().print("All finished.");
    } else {
      response.getWriter().print("Upload");
    } 
  }
  
  protected void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
    int resumableChunkNumber = getResumableChunkNumber(request);
    ResumableInfo info = getResumableInfo(request);
    if (info.uploadedChunks.contains(new ResumableInfo.ResumableChunkNumber(resumableChunkNumber))) {
      response.getWriter().print("Uploaded.");
    } else {
      response.setStatus(404);
    } 
  }
  
  private int getResumableChunkNumber(HttpServletRequest request) {
    return HttpUtils.toInt(request.getParameter("resumableChunkNumber"), -1);
  }
  
  private ResumableInfo getResumableInfo(HttpServletRequest request) throws ServletException {
    String base_dir = UPLOAD_DIR;
    int resumableChunkSize = HttpUtils.toInt(request.getParameter("resumableChunkSize"), -1);
    long resumableTotalSize = HttpUtils.toLong(request.getParameter("resumableTotalSize"), -1L);
    String resumableIdentifier = request.getParameter("resumableIdentifier");
    String resumableFilename = request.getParameter("resumableFilename");
    String resumableRelativePath = request.getParameter("resumableRelativePath");
    (new File(base_dir)).mkdir();
    String resumableFilePath = (new File(base_dir, resumableFilename)).getAbsolutePath() + ".temp";
    ResumableInfoStorage storage = ResumableInfoStorage.getInstance();
    ResumableInfo info = storage.get(resumableChunkSize, resumableTotalSize, resumableIdentifier, resumableFilename, resumableRelativePath, resumableFilePath);
    if (!info.vaild()) {
      storage.remove(info);
      throw new ServletException("Invalid request params.");
    } 
    return info;
  }
}
